@@ -11,7 +11,7 @@
 (use-modules (nyacc lang c99 parser))
 (use-modules (nyacc lang c99 pprint))	; pretty-print-c99
 (use-modules (nyacc lang c99 util1))	; remove-inc-trees
-(use-modules (nyacc lang c99 util2))	; tree->udict
+(use-modules (nyacc lang c99 munge))	; tree->udict
 (use-modules (nyacc lang util))		; move-if-changed
 ;;(use-modules (nyacc lalr))
 (use-modules (nyacc util))
@@ -20,7 +20,7 @@
 
 ;; test def parser: show parse tree
 (when #t
-  (let* ((file "exam.d/ex01.c")
+  (let* ((file "c99-exam/ex01.c")
 	 (defs '("arch=x86_64"))
 	 (incs  '("exam.d"))
 	 (parse (lambda () (parse-c99
