@@ -1,6 +1,6 @@
 ;;; nyacc/lang/modelica/mach.scm
 ;;;
-;;; Copyright (C) 2015,2016 Matthew R. Wette
+;;; Copyright (C) 2015-2017 Matthew R. Wette
 ;;;
 ;;; This program is free software: you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by 
@@ -25,34 +25,33 @@
 	    parse-mo)
   #:use-module (nyacc lalr)
   #:use-module (nyacc lex)
+  #:use-module (nyacc parse)
   #:use-module (nyacc lang util)
   #:use-module (ice-9 pretty-print)
   )
 
-(fmterr "modelica/pgen.scm grammar conflicts:\n")
-(fmterr "  SR: primary => \"initial\"\n")
-(fmterr "  SR: primary => \"end\"\n")
-
 (define modelica-spec
   (lalr-spec
-   (notice (string-append "Copyright 2016 Matthew R. Wette" lang-crn-lic))
+   (notice (string-append "Copyright 2016-2017 Matthew R. Wette" lang-crn-lic))
    (start stored-definition)
    (grammar
     
     ;; B.2.1
     (stored-definition
      ( ($$ `(stored-defn)))
-     (stored-definition-1 stored-definition-2 ($$ `(stored-defn FIX)))
+     (stored-definition-1 stored-definition-2 ($$ `(stored-defn ,$1 ,$2)))
      (stored-definition-2 ($$ (tl->list $1)))
      )
     (stored-definition-1
-     ("within" name ";")
-     ("within" ";")
+     ("within" name ";" ($$ `(within ,$2)))
+     ("within" ";" ($$ '(within)))
      )
     (stored-definition-2
-     ("final" class-definition ";" ($$ `(FIX01)))
-     (class-definition ";" ($$ (make-tl 'defn-list $1)))
-     (stored-definition-2 "final" class-definition ";" ($$ `(FIX01)))
+     ("final" class-definition ";"
+      ($$ (make-tl 'class-defn-list (sx+attr* $2 'final "yes"))))
+     (class-definition ";" ($$ (make-tl 'class-defn-list $1)))
+     (stored-definition-2 "final" class-definition ";"
+			  ($$ (tl-append $3 (sx+attr* $3 'final "yes"))))
      (stored-definition-2 class-definition ";" ($$ (tl-append $1 $2)))
      )
 
@@ -106,43 +105,47 @@
 		(list $1 $2 $3)))
      ("extends" ident class-modification string-comment composition "end" ident
       ($$ (if (not (string=? (cadr $2) (cadr $7)))
-	      (error "ident's don't match"))
+	      (throw 'mo-error "ident's don't match"))
 	  (list '(@ extends . "yes") $2 $3 $4 $5)))
      ("extends" ident string-comment composition "end" ident
       ($$ (if (not (string=? (cadr $2) (cadr $6)))
-	      (error "ident's don't match"))
+	      (throw 'mo-error "ident's don't match"))
 	  (list '(@ extends . "yes") $2 $3 $4)))
      )
 
     (short-class-specifier
-     (ident "=" base-prefix name array-subscripts class-modification comment
+     (ident "=" base-prefix type-specificer array-subscripts
+	    class-modification comment
 	    ($$ (list $1 `(is ,$3 ,$4 ,$5 ,$6 ,$7))))
-     (ident "=" base-prefix name array-subscripts comment)
-     (ident "=" base-prefix name class-modification comment)
-     (ident "=" base-prefix name comment)
-     (ident "=" "enumeration" "(" filler-1 ")" comment)
+     (ident "=" base-prefix type-specifier array-subscripts comment)
+     
+     (ident "=" base-prefix type-specifier class-modification comment)
+     (ident "=" base-prefix type-specifier comment)
+     (ident "=" "enumeration" "(" enum-list ")" comment)
+     (ident "=" "enumeration" "(" ":" ")" comment)
      )
-    (filler-1 () (enum-list) (":"))
 
     (der-class-specifier
-     (ident "=" "der" "(" name "," der-class-specifier-1 ")" comment)
+     (ident "=" "der" "(" type-specifier "," der-class-specifier-1 ")" comment
+	    ($$ `(der-class-specifier ... (tl->list $7))))
      )
     (der-class-specifier-1 ;; or ident-colon-list
-     (ident)
-     (der-class-specifier-1 ";" ident)
+     (ident ($$ (make-tl 'ident-list $1)))
+     (der-class-specifier-1 ";" ident ($$ (tl-append $1 $3)))
      )
 
-    (base-prefix () (type-prefix))
-    ;;(base-prefix (type-prefix)) ;; type-prefix has no empty now
+    (base-prefix ("input") ("output"))
 
     (enum-list
-     (enumeration-literal)
-     (enum-list "," enumeration-literal))
+     (enumeration-literal ($$ (make-tl 'enum-list $1)))
+     (enum-list "," enumeration-literal ($$ (tl-append $1 $3))))
 
     (enumeration-literal
      (ident comment)
      )
 
+    ;; ===================== update to v3.4 stopped here ======================
+
     (composition
      (element-list composition-1-list external-part opt-annotation)
      (element-list composition-1-list opt-annotation)
@@ -682,4 +685,22 @@
 
 (define (parse-mo) (modelica-parser (gen-mod-lexer)))
 
+;; === automaton file generator =========
+
+(define (gen-modelica-files . rest)
+  (define (lang-dir path)
+    (if (pair? rest) (string-append (car rest) "/" path) path))
+  (define (xtra-dir path)
+    (lang-dir (string-append "mach.d/" path)))
+
+  (write-lalr-actions matlab-mach (xtra-dir "moact.scm.new"))
+  (write-lalr-tables matlab-mach (xtra-dir "motab.scm.new"))
+  (let ((a (move-if-changed (xtra-dir "moact.scm.new")
+			    (xtra-dir "moact.scm")))
+	(b (move-if-changed (xtra-dir "motab.scm.new")
+			    (xtra-dir "motab.scm"))))
+    (when #f ;;(or a b) 
+      (system (string-append "touch " (lang-dir "parser.scm"))))
+    (or a b)))
+
 ;; --- last line ---
