@@ -6,23 +6,29 @@
 ;;; or any later version published by the Free Software Foundation.  See
 ;;; the file COPYING included with the nyacc distribution.
 
-;; runtime utilities for the parsers -- needs work
+;; runtime utilities for the parsers
 
 (define-module (nyacc lang util)
   #:export (lang-crn-lic 
 	    report-error
-	    push-input pop-input reset-input-stack
+	    *input-stack* push-input pop-input reset-input-stack
 	    make-tl tl->list ;; rename?? to tl->sx for sxml-expr
 	    tl-append tl-insert tl-extend tl+attr tl+attr*
-	    sx-tag sx-attr sx-tail sx-ref sx-cons* sx-list
-	    sx-attr-ref sx-has-attr? sx-set-attr! sx-set-attr* sx+attr*
+	    sx-tag sx-attr sx-tail sx-length sx-ref sx-ref* sx-cons* sx-list
+	    sx-attr-ref sx-has-attr? sx-attr-set! sx-attr-set* sx+attr*
 	    sx-find
 	    ;; for pretty-printing
 	    make-protect-expr make-pp-formatter make-pp-formatter/ugly
 	    ;; for ???
 	    move-if-changed
-	    fmterr)
-  #:use-module ((srfi srfi-1) #:select(find))
+	    cintstr->scm
+	    sferr pperr
+	    ;; depracated
+	    sx-set-attr! sx-set-attr*
+	    )
+  #:use-module ((srfi srfi-1) #:select (find fold))
+  ;; #:use-module ((sxml xpath) #:select (sxpath)) ;; see sx-find below
+  #:use-module (ice-9 pretty-print)
   )
 
 ;; This is a generic copyright/licence that will be printed in the output
@@ -33,8 +39,10 @@ This software is covered by the GNU GENERAL PUBLIC LICENCE, Version 3,
 or any later version published by the Free Software Foundation.  See
 the file COPYING included with the this distribution.")
 
-(define (fmterr fmt . args)
+(define (sferr fmt . args)
   (apply simple-format (current-error-port) fmt args))
+(define (pperr exp . kw-args)
+  (apply pretty-print exp (current-error-port) kw-args))
 
 ;; @deffn {Procedure} report-error fmt args
 ;; Report an error: to stderr, providing file and line num info, and add nl.
@@ -46,7 +54,7 @@ the file COPYING included with the this distribution.")
 
 ;; === input stack =====================
 
-(define *input-stack* (make-fluid '()))
+(define *input-stack* (make-fluid #f))
 
 (define (reset-input-stack)
   (fluid-set! *input-stack* '()))
@@ -55,6 +63,7 @@ the file COPYING included with the this distribution.")
   (let ((curr (current-input-port))
 	(ipstk (fluid-ref *input-stack*)))
     (fluid-set! *input-stack* (cons curr ipstk))
+    ;;(sferr "~S pu=>\n" (length ipstk))
     (set-current-input-port port)))
 
 ;; Return #f if empty
@@ -62,6 +71,7 @@ the file COPYING included with the this distribution.")
   (let ((ipstk (fluid-ref *input-stack*)))
     (if (null? ipstk) #f
 	(begin
+	  ;;(sferr "~S <=po\n" (length ipstk))
 	  (set-current-input-port (car ipstk))
 	  (fluid-set! *input-stack* (cdr ipstk))))))
 
@@ -167,10 +177,29 @@ the file COPYING included with the this distribution.")
 ;; attributea are `invisible'. For example, @code{'(elt (@abc) "d")}
 ;; is an sx of length two: the tag @code{elt} and the payload @code{"d"}.
 
+(define (sxml-expr? sx)
+  (and (pair? sx) (symbol? (car sx)) (list? sx)))
+
+;; @deffn {Procedure} sx-length sx => <int>
+;; Return the length, don't include attributes, but do include tag
+;; @end deffn
+(define (sx-length sx)
+  (let ((ln (length sx)))
+    (cond
+      ((zero? ln) 0)
+      ((= 1 ln) 1)
+      ((not (pair? (cadr sx))) ln)
+      ((eq? '@ (caadr sx)) (1- ln))
+      (else ln))))
+
 ;; @deffn {Procedure} sx-ref sx ix => item
 ;; Reference the @code{ix}-th element of the list, not counting the optional
 ;; attributes item.  If the list is shorter than the index, return @code{#f}.
+;; [note to author: The behavior to return @code{#f} if no elements is not
+;; consistent with @code{list-ref}.  Consider changing it.  Note also there
+;; is never a danger of an element being @code{#f}.]
 ;; @example
+;; (sx-ref '(abc 1) => #f
 ;; (sx-ref '(abc "def") 1) => "def"
 ;; (sx-ref '(abc (@ (foo "1")) "def") 1) => "def"
 ;; @end example
@@ -179,11 +208,21 @@ the file COPYING included with the this distribution.")
   (define (list-xref l x) (if (> (length l) x) (list-ref l x) #f))
   (cond
    ((zero? ix) (car sx))
+   ((null? (cdr sx)) #f)
    ((and (pair? (cadr sx)) (eqv? '@ (caadr sx)))
     (list-xref sx (1+ ix)))
    (else
     (list-xref sx ix))))
 
+;; @deffn {Procedure} sx-ref* sx ix1 ix2 ... => item
+;; Equivalent to
+;; @example
+;; (((sx-ref (sx-ref sx ix1) ix2) ...) ...)
+;; @end example
+;; @end deffn
+(define (sx-ref* sx . args)
+  (fold (lambda (ix sx) (sx-ref sx ix)) sx args))
+
 ;; @deffn {Procedure} sx-tag sx => tag
 ;; Return the tag for a tree
 ;; @end deffn
@@ -193,17 +232,47 @@ the file COPYING included with the this distribution.")
 ;; @deffn {Procedure} sx-cons* tag (attr|#f)? ... => sx
 ;; @deffnx {Procedure} sx-list tag (attr|#f)? ... => sx
 ;; Generate the tag and the attr list if it exists.  Note that
+;; The following are equivalent:
+;; @example
+;; (sx-cons* tag attr elt1 elt2 '())
+;; (sx-list tag attr elt1 elt2)
+;; @end example
+;; @noindent
+;; NEW IMPLEMENTATION: now any field that is #f will be skipped
 ;; @end deffn
 (define (sx-cons* tag . rest)
-  (cond
-   ((null? rest) (list tag))
-   ((not (car rest)) (apply cons* tag (cdr rest)))
-   (else (apply cons* tag rest))))
+  (if (null? rest) (error "expecing tail"))
+  ;;(cond
+  ;; ((null? rest) (list tag))
+  ;; ((not (car rest)) (apply cons* tag (cdr rest)))
+  ;; (else (apply cons* tag rest)))
+  (cons tag
+	(let iter ((items rest))
+	  (if (null? (cdr items)) (car items)
+	      (if (car items)
+		  (cons (car items) (iter (cdr items)))
+		  (iter (cdr items)))))))
+
 (define (sx-list tag . rest)
-  (cond
-   ((null? rest) (list tag))
-   ((not (car rest)) (apply list tag (cdr rest)))
-   (else (apply list tag rest))))
+  ;;(cond
+  ;; ((null? rest) (list tag))
+  ;; ((not (car rest)) (apply list tag (cdr rest)))
+  ;; (else (apply list tag rest))))
+  (cons tag
+	(let iter ((items rest))
+	  (if (null? items) '()
+	      (if (car items)
+		  (cons (car items) (iter (cdr items)))
+		  (iter (cdr items)))))))
+
+;;. maybe change to case-lambda to accept count
+;; @example
+;; (sx-repl-tail (tag (@ ...) (orig-elt ...)) (repl-elt ...))
+;; => 
+;; (sx-repl-tail (tag (@ ...) (repl-elt ...)))
+;; @end example
+(define (sx-repl-tail sexp tail)
+  (sx-cons* (sx-tag sexp) (sx-attr sexp) tail))
 
 ;; @deffn {Procedure} sx-tail sx [ix] => (list)
 ;; Return the ix-th tail starting after the tag and attribut list, where
@@ -250,28 +319,30 @@ the file COPYING included with the this distribution.")
 ;; @end deffn
 (define (sx-attr-ref sx key)
   (and=> (sx-attr sx)
-	 (lambda (attr)
-	   (and=> (assq-ref (cdr attr) key) car))))
+	 (lambda (attr) (and=> (assq-ref (cdr attr) key)
+			       car))))
 
-;; @deffn {Procedure} sx-set-attr! sx key val
+;; @deffn {Procedure} sx-attr-set! sx key val
 ;; Set attribute for sx.  If no attributes exist, if key does not exist,
 ;; add it, if it does exist, replace it.
 ;; @end deffn
-(define (sx-set-attr! sx key val)
+(define (sx-attr-set! sx key val)
   (if (sx-has-attr? sx)
       (let ((attr (cadr sx)))
 	(set-cdr! attr (assoc-set! (cdr attr) key (list val))))
       (set-cdr! sx (cons `(@ (,key ,val)) (cdr sx))))
   sx)
+(define sx-set-attr! sx-attr-set!)
 
-;; @deffn {Procedure} sx-set-attr* sx key val [key val [key ... ]]
+;; @deffn {Procedure} sx-attr-set* sx key val [key val [key ... ]]
 ;; Generate sx with added or changed attributes.
 ;; @end deffn
-(define (sx-set-attr* sx . rest)
+(define (sx-attr-set* sx . rest)
   (let iter ((attr (or (and=> (sx-attr sx) cdr) '())) (kvl rest))
     (cond
      ((null? kvl) (cons* (sx-tag sx) (cons '@ (reverse attr)) (sx-tail sx 1)))
      (else (iter (cons (list (car kvl) (cadr kvl)) attr) (cddr kvl))))))
+(define sx-set-attr* sx-attr-set*)
 
 ;; @deffn {Procedure} sx+attr* sx key val [key val [@dots{} ]] => sx
 ;; Add key-val pairs. @var{key} must be a symbol and @var{val} must be
@@ -285,13 +356,27 @@ the file COPYING included with the this distribution.")
     (cons* (sx-tag sx) (cons '@ attrs)
 	   (if (sx-has-attr? sx) (cddr sx) (cdr sx)))))
 
-;; @deffn {Procedure} sx-find tag sx => ((tag ...) (tag ...))
+;; @deffn {Procedure} sx-find tag sx => (tag ...)
+;; @deffnx {Procedure} sx-find path sx => (tag ...)
+;; In the first form @var{tag} is a symbolic tag in the first level.
 ;; Find the first matching element (in the first level).
+;; In the second form, the argument @var{path} is a pair.  Apply sxpath
+;; and take it's car,
+;; if found, or return @code{#f}, like lxml's @code{tree.find()} method.
+;; @* NOTE: the path version is currently disabled, to remove dependence
+;; on the module @code{(sxml xpath)}.
 ;; @end deffn
-(define (sx-find tag sx)
-  (find (lambda (node)
-	  (and (pair? node) (eqv? tag (car node))))
-	sx))
+(define (sx-find tag-or-path sx)
+  (cond
+   ((symbol? tag-or-path)
+    (find (lambda (node)
+	    (and (pair? node) (eqv? tag-or-path (car node))))
+	  sx))
+   #;((pair? tag-or-path)
+    (let ((rez ((sxpath tag-or-path) sx)))
+      (if (pair? rez) (car rez) #f)))
+   (else
+    (error "expecting first arg to be tag or sxpath"))))
 
 ;;; === pp ==========================
 ;; @section Pretty-Print and Other Utility Procedures
@@ -346,7 +431,42 @@ the file COPYING included with the this distribution.")
 	((=) (assc? op))
 	(else #f)))))
 
-;; @deffn {Procedure} make-pp-formatter [port] [#:per-line-prefix ""] => fmtr
+;; @deffn {Procedure} expand-tabs str [col]
+;; Expand tabs where the string @var{str} starts in column @var{col}
+;; (default 0). 
+;; @end deffn
+(define* (expand-tabs str #:optional (col 0))
+
+  (define (fill-tab col chl)
+    (let iter ((chl (if (zero? col) (cons #\space chl) chl))
+	       (col (if (zero? col) (1+ col) col)))
+      (if (zero? (modulo col 8)) chl
+	  (iter (cons #\space chl) (1+ col)))))
+
+  (define (next-tab-col col) ;; TEST THIS !!!
+    ;;(* 8 (quotient (+ 9 col) 8))) ???
+    (* 8 (quotient col 8)))
+
+  (let ((strlen (string-length str)))
+    (let iter ((chl '()) (col col) (ix 0))
+      (if (= ix strlen) (list->string (reverse chl))
+	  (let ((ch (string-ref str ix)))
+	    (case ch
+	      ((#\newline)
+	       (iter (cons ch chl) 0 (1+ ix)))
+	      ((#\tab)
+	       (iter (fill-tab col chl) (next-tab-col col) (1+ ix)))
+	      (else
+	       (iter (cons ch chl) (1+ col) (1+ ix)))))))))
+
+;; @deffn {Procedure} make-pp-formatter [port] <[options> => fmtr
+;; Options
+;; @table @code
+;; @item #:per-line-prefix
+;; string to prefix each line
+;; @item #:width
+;; Max width of output.  Default is 79 columns.
+;; @end itemize
 ;; @example
 ;; (fmtr 'push) ;; push indent level
 ;; (fmtr 'pop)  ;; pop indent level
@@ -354,28 +474,28 @@ the file COPYING included with the this distribution.")
 ;; @end example
 ;; @end deffn
 (define* (make-pp-formatter #:optional (port (current-output-port))
-			    #:key per-line-prefix)
-  (letrec
-      ((maxcol (- 78 (if per-line-prefix (string-length per-line-prefix) 0)))
-       ;;(maxcol 78)
+			    #:key per-line-prefix (width 79) (basic-offset 2))
+  (letrec*
+      ((pfxlen (string-length (expand-tabs (or per-line-prefix ""))))
+       (maxcol (- width (if per-line-prefix pfxlen 0)))
        (maxind 36)
        (column 0)
        (ind-lev 0)
        (ind-len 0)
        (blanks "                                            ")
        (ind-str (lambda () (substring blanks 0 ind-len)))
-       (cnt-str (lambda () (substring blanks 0 (+ 4 ind-len))))
+       (cnt-str (lambda () (substring blanks 0 (+ basic-offset 2 ind-len))))
        ;;(sf-nl (lambda () (newline) (set! column 0)))
 
        (push-il
 	(lambda ()
 	  (set! ind-lev (min maxind (1+ ind-lev)))
-	  (set! ind-len (* 2 ind-lev))))
+	  (set! ind-len (* basic-offset ind-lev))))
 
        (pop-il
 	(lambda ()
 	  (set! ind-lev (max 0 (1- ind-lev)))
-	  (set! ind-len (* 2 ind-lev))))
+	  (set! ind-len (* basic-offset ind-lev))))
 
        (inc-column!
 	(lambda (inc)
@@ -388,6 +508,17 @@ the file COPYING included with the this distribution.")
        (sf
 	(lambda (fmt . args)
 	  (let* ((str (apply simple-format #f fmt args))
+		 (str (if (and (zero? column) per-line-prefix)
+			  (begin
+			    (when #f ;;(char=? #\tab (string-ref str 0))
+			      (sferr "expand-tabs (pfxlen=~S)\n" pfxlen)
+			      (sferr "~A\n" str)
+			      (sferr "~A~A\n\n" per-line-prefix
+				     (expand-tabs str pfxlen))
+			      )
+			    (expand-tabs str pfxlen)
+			    )
+			  str))
 		 (len (string-length str)))
 	    (cond
 	     ((zero? column)
@@ -492,4 +623,37 @@ the file COPYING included with the this distribution.")
     (simple-format (current-error-port) "move-if-changed: no write access\n")
     #f)))
 
+;; @deffn {Procedure} cintstr->scm str => #f|str
+;; Convert a C string for a fixed type to a Scheme string.
+;; If not identified as a C int, then return @code{#f}.
+;; TODO: add support for character literals (and unicode?).
+;; @end deffn
+(define cs:dig (string->char-set "0123456789"))
+(define cs:hex (string->char-set "0123456789ABCDEFabcdef"))
+(define cs:oct (string->char-set "01234567"))
+(define cs:long (string->char-set "lLuU"))
+(define (cintstr->scm str)
+  ;; dl=digits, ba=base, st=state, ix=index
+  ;; 0: "0"->1, else->2
+  ;; 1: "x"->(base 16)2, else->(base 8)2
+  ;; 2: "0"-"9"->(cons ch dl), else->3:
+  ;; 3: "L","l","U","u"->3, eof->(cleanup) else->#f
+  (let ((ln (string-length str)))
+    (let iter ((dl '()) (bx "") (cs cs:dig) (st 0) (ix 0))
+      (if (= ix ln)
+	  (if (null? dl) #f (string-append bx (list->string (reverse dl))))
+	  (case st
+	    ((0) (iter (cons (string-ref str ix) dl) bx cs
+		       (if (char=? #\0 (string-ref str ix)) 1 2)
+		       (1+ ix)))
+	    ((1) (if (char=? #\x (string-ref str ix))
+		     (iter '() "#x" cs:hex 2 (1+ ix))
+		     (iter '() "#o" cs:oct 2 ix)))
+	    ((2) (if (char-set-contains? cs (string-ref str ix))
+		     (iter (cons (string-ref str ix) dl) bx cs st (1+ ix))
+		     (if (char-set-contains? cs:long (string-ref str ix))
+			 (iter dl bx cs 3 (1+ ix))
+			 #f)))
+	    ((3) #f))))))
+
 ;;; --- last line ---
